/*
 * Copyright (c) 2018 Alibaba Group Holding Ltd. All rights reserved.
 */

'use strict';

const DBusProxy = require('./DBusProxy');
const Try = require('../../fc/try');

function CredProxy() {
  this._dbusCred = new DBusProxy(
    'iot.gateway.credential',
    '/iot/gateway/credential',
    'iot.gateway.credential');
}

CredProxy.prototype.getCredential = function (roleArn, callback) {
  if (roleArn === undefined || roleArn === null) {
    throw new Error(`"roleArn" is a required parameter.`);
  }
  this._dbusCred.callServiceAsync('getCredential', roleArn, function (err, data) {
    if (err) {
      err = new Error(`Can not get credential for ${roleArn} due to ${err}.`);
    } else {
      var credTry = Try.JSON.parse(data);
      if (credTry.error) {
        err = credTry.error;
      } else {
        if (credTry.value.code === 0) {
          data = credTry.value.data;
        } else {
          var message = credTry.value.message ? credTry.value.message.cause :
            `Can not get credential for ${roleArn}.`;
          err = new Error(message);
        }
      }
    }
    if (callback) {
      callback(err, data);
    }
  });
};

module.exports = CredProxy;