'use strict';

const DEFAULT_LEVEL = 0;
const DEFAULT_TAG = "iot-gravity";
const DEFAULT_FILE = "proxy/LogSystemProxy.js";
const DEFAULT_FUNC = "trace";
const DEFAULT_LINE = 0;

var LogSystemProxy = function () {
    /**
     * 给log系统打印一个log
     * @param level log等级
     *           LOG_LEVEL_DEBUG = 0,
     *           LOG_LEVEL_INFO = 1,
     *           LOG_LEVEL_WARN = 2,
     *           LOG_LEVEL_ERR = 3,
     *           LOG_LEVEL_FATAL = 4
     * @param tag TAG
     * @param file 文件名
     * @param func 函数名
     * @param line 文件中的行号
     * @param log log信息
     */
    return {
        trace: function (level, tag, file, func, line, log) {
            var server = require("./EdgeSideServer");
            // 为方便，单独一个string默认为log，其它使用默认参数
            if (arguments.length === 1 && typeof arguments[0] === 'string') {
                server.traceLog(DEFAULT_LEVEL, DEFAULT_TAG, DEFAULT_FILE, DEFAULT_FUNC, DEFAULT_LINE, arguments[0] + "\n");
            }
            /**
             * 不满足条件的log都不发出去，因为dbus读这些数据要按顺序读，
             * 如果数据类型不对，那么dbus会出错。
             * 所以，设置DEFAULT值。
             */
            else if ((typeof arguments[0] !== 'number') || (typeof arguments[4] !== 'number') ||
                (typeof arguments[1] !== 'string') || (typeof arguments[2] !== 'string') ||
                (typeof arguments[3] !== 'string') || (typeof arguments[5] !== 'string')) {
                // 不符合数据类型条件的log，直接忽略
                // console.log("参数检查失败");
            } else {
                server.traceLog(level, tag, file, func, line, log + "\n");
            }
        }
    }
};

module.exports = new LogSystemProxy;