'use strict';
/*var config = [{
 "jobId": 0,
 "ackCode" : 10201,
 "path": "/workspace/xiaomin.wxm/Projects/iot-gravity/loader/test",
 "type" : "xxx",
 "executor" : "NodeEngine",
 "jobstatus": ""
},
 {
   "jobId": 1,
   "ackCode" : 10201,
   "path": "",
   "type" : "xxx",
   "executor" : "NodeEngine",
   "jobstatus": ""
 }];*/
var fs = require("fs");
const constant = require("../../utils/constant");
const path = require("path");
const broker = require('../broker');
const jobmProxy = require('../dbus/dbus-proxy/JobManagerProxy.js');
var Trigger = require('./Trigger');
const Log = require("../../utils/log");
var dmp = require("../dbus/dbus-proxy/DMPProxy");
var scheduler = require("../../scheduler/scheduler");
const TAG = "[FC_TriggerManager] ";
var _instance = null;//定义初始化_instance
var triggerScripts = [];
var exeBegin = false;
const {PROJECT_PATH} = require('../../const');
const dirname = path.join(PROJECT_PATH, 'components/fc');

var TriggerManager = function (functionService) {
  Log.i(TAG, "TriggerManager new begin...");
  _instance = this;
  this.functionService = functionService;
  this.configPath = path.join(constant.FOTA_FC, "configuration.json");
  console.log(" configuration path: " + this.configPath);
  this.config = this.readConfiguration();
  this.receiveJobSignal();
  // this.receiveEvent(); filter replace
  this.receiveFilterSignal();
  this.interval = setInterval(() => {
    this.tryStartTrigger();
  }, 1000);
  console.log(TAG + "TriggerManager new end");
};
TriggerManager.ACTION_ADD_JOB = 10201; // 增加场景
TriggerManager.ACTION_DELETE_JOB = 10204; // 删除场景
TriggerManager.COMMAND_DELETE_JOB = 10211;

TriggerManager.EXEUTE_RESULT = {
  JOB_DELETE_SUCCESS: 0,
  JOB_NOT_RUN: 1,
  JOB_SUCCEED_RUNNING: 2,
  JOB_EXECUTE_END: 3,
  JOB_EXECUTE_FAILED: 4,
  JOB_EXECUTE_HANG: 5
};

TriggerManager.getInstance = function () {
  if (_instance != null) {
    return _instance;
  }
  throw new Error('TriggerManager has not been constructed.');
};

TriggerManager.prototype.tryStartTrigger = function () {
  Log.i(TAG, "tryStartTrigger called.");
  if (dmp.isOnline()) {
    clearInterval(this.interval);
    setTimeout(() => {
      Log.i(TAG, "dmp service is online");
      exeBegin = true;
      this.allTrigger(this.config);
    }, 1000);
  } else {
    Log.i(TAG, "dmp service is offline");
  }
};

/**
 * 要执行的job 为configuration.json 单个元素
 * @param  {[Object]} 单条config，格式如下：
 * {
    "jobId": 0,
    "ackCode" : 10201,
    "path": "",
    "type" : "xxx",
    "executor" : "yyy",
    "jobstatus": ""  //比原来job多的一个字段,
    "groupId" : "xxx",
    "parentGroupIds" : ["901", "hotel"],
    "sceneId" : ""
    }
 */
TriggerManager.prototype.trigger = function (job) {
  Log.i(TAG, "trigger begin");
  if (!exeBegin) {
    Log.job(TAG, "exeBegin = " + exeBegin + " will wait");
    return;
  }
  var ackCode = job.ackCode;
  Log.i(TAG, "trigger begin...jobid = " + job.jobId + " job.ackCode = " + job.ackCode + "\n" + "triggerScripts.size = " + triggerScripts.length);
  var self = this;
  if (ackCode === TriggerManager.ACTION_DELETE_JOB) {
    Log.i(TAG, "ACTION_DELETE_JOB trigger");
    this.deleteJob(job, true);
  }

  if (ackCode === TriggerManager.COMMAND_DELETE_JOB)
  {
    Log.i(TAG, "COMMAND_DELETE_JOB trigger");
    this.deleteJob(job, true);
  }
  if (ackCode === TriggerManager.ACTION_ADD_JOB) {
    Log.i(TAG, "ACTION_ADD_JOB trigger begin");
    new Trigger(job, this.functionService, function (err, triggerObject) {
      Log.i(TAG, "registerTrigger err = " + err);
      var checkError = triggerObject.checkError();
      var status;
      var message;
      if (checkError) {
        status = TriggerManager.EXEUTE_RESULT.JOB_EXECUTE_FAILED;
        message = checkError;
        Log.e(TAG, checkError);
        self.sendJobStatus(job.ackId, job.ackCode, job.jobId, status, job.executor, message, 0, job.sceneId);
        return;
      }
      Log.i(TAG, "hasTrigger = " + triggerObject.hasTrigger());
      Log.i(TAG, "self.isTriggeredJob(job) = " + self.isTriggeredJob(job));

      if (triggerObject && !self.isTriggeredJob(job)) {
        //插入相同jobId进来后不会再执行监听后面的jobId对应的trigger
        triggerObject.startCrontab();
        triggerScripts.push(triggerObject);
      }
      try {
        if ((triggerObject.config)[constant.CONFIG_CONFIG][constant.CONFIG_RUNMODE] === constant.CONFIG_RUNMODE_LONGLIVE) {
          self.functionService.startFunction(triggerObject.functionId, function (err) {
            if (err) {
              Log.e(TAG, `Start function ${triggerObject.functionId} failed: ${err}`);
            } else {
              Log.e(TAG, `Start function ${triggerObject.functionId} successfully.`);
            }
          })
        }
      } catch (err) {
        Log.e(TAG, "err = " + err);
      }
      status = TriggerManager.EXEUTE_RESULT.JOB_SUCCEED_RUNNING;
      message = "execute success";
      self.sendJobStatus(job.ackId, job.ackCode, job.jobId, status, job.executor, message, 0, job.sceneId);
    });
    Log.i(TAG, "trigger func triggerScripts size = " + triggerScripts.length);
  }
};

TriggerManager.prototype.deleteJob = function (job, notifyJobManager) {
  Log.i(TAG, "ACTION_DELETE_JOB execute notifyJobManager =", notifyJobManager);
  if (this.isValidJob(job)) {
    if (this.isTriggeredJob(job)) {
      this.deleteTriggerScript(job, notifyJobManager);
    } else {
      this.deleteNoTriggerScript(job, notifyJobManager, true);
    }
  } else {
    Log.e(TAG, `ACTION_DELETE_JOB id : ${job.jobId} not exists`);
    if (notifyJobManager) {
      this.sendJobStatus(job.ackId, job.ackCode, job.jobId, TriggerManager.EXEUTE_RESULT.JOB_DELETE_SUCCESS, job.executor, `ACTION_DELETE_JOB id : ${job.jobId} not exists`, 0, job.sceneId);
    }
  }
  Log.i(TAG, "DELETE_JOB end ackCode= " + job.ackCode);
};

TriggerManager.prototype.deleteNoTriggerScript = function (job, notifyJobManager, related) {
  var self = this;
  var deleteIndex = [];
  var functionIds = [];
  /*if(related) {
      DataCenter.clearThing();
  }*/
  this.config.forEach(function (item, index) {
    // console.log(TAG + "forEach jobId = " + item.jobId + " ackCode = " + item.ackCode);
    if (item.jobId === job.jobId) {
      deleteIndex.push(index);
      functionIds.push(item.functionId);
    }
    /*else if(related) {
               self.trigger(item);
           }*/
  });
  functionIds.forEach(function (functionId) {
    self.functionService.stopFunction(functionId, function () {/* Ignore the result. */});
  });
  for (let i = deleteIndex.length - 1; i >= 0; i--) {
    this.config.splice(deleteIndex[i], 1);
  }
  this.writeConfiguration(JSON.stringify(this.config, null, 4));
  if (notifyJobManager) {
    self.sendJobStatus(job.ackId, TriggerManager.ACTION_DELETE_JOB, job.jobId, TriggerManager.EXEUTE_RESULT.JOB_DELETE_SUCCESS, job.executor, "delete success", 0, job.sceneId);
  }
};

TriggerManager.prototype.deleteTriggerScript = function (job, notifyJobManager) {
  removeOneTrigger(job.jobId);
  this.deleteNoTriggerScript(job, notifyJobManager, false);
};

/**
 * 发送script的执行状态给jobmanager
 *
 * @param  {[type]} jobstatus [job的执行状态，格式如下：
 * {
        "ackCode" : int
        "jobId" :   long
        "jobStatus" : int // 1：未运行 2:运行中 3:执行完成 4:执行失败 5:被挂起
        "executor" : string
        "result" : string
    }
 ]
 * @return {[type]}           [description]
 */
TriggerManager.prototype.sendJobStatus = function (ackId, ackCode, jobId, jobStatus, executor, result, category, sceneId) {
  if (typeof result !== 'string' && result) {
    result = result.toString();
  }
  var object = {};
  object["ackId"] = ackId;
  object["ackCode"] = ackCode;
  object["jobId"] = jobId;
  object["jobStatus"] = jobStatus;
  object["executor"] = executor;
  object["result"] = result;
  object["category"] = category;
  object["sceneId"] = sceneId;
  var tag;
  switch (jobStatus) {
    case 2 : {
      tag = "[FC_Deploy_Result][Succeeded]";
      break;
    }
    case 4 : {
      tag = "[FC_Deploy_Result][Failed]";
      break;
    }
    case 0 : {
      tag = "[FC_RESET_Result][Succeeded]";
      break;
    }
    default : {
      tag = "[FC_Deploy_Result][Unknown]";
      break;
    }
  }
  if (sceneId && sceneId !== "") {
    Log.m(tag, `[${sceneId}]`, 'result = ' + result);
  } else {
    Log.m(tag, 'result = ' + result);
  }
  jobmProxy.sendJobStatus(object);
};

TriggerManager.prototype.allTrigger = function (config) {
  if (!config) {
    Log.e(TAG, "allTrigger config null");
    return;
  }
  Log.i(TAG + "allTrigger begin...");
  var self = this;
  config.forEach(function (item) {
    self.trigger(item);
  });
};

TriggerManager.prototype.isTriggeredJob = function (job) {
  return triggerScripts.some(script => script.jobId === job.jobId);
};

TriggerManager.prototype.isValidJob = function (job) {
  return this.config.some(item => item.jobId === job.jobId);
};

TriggerManager.prototype.receiveJobSignal = function () {
  console.log(TAG + "begin receiveSignal ...");
  broker.subscribe("NodeEngine_Event", onScriptEvent);
  broker.subscribe("NodeEngine_Config_Event", onGroupConfigUpdate);
};

TriggerManager.prototype.receiveFilterSignal = function () {
  broker.subscribe("Filter_Event", onFilterEvent);
};

TriggerManager.prototype.receiveEvent = function () {
  broker.subscribe('propertiesChanged', onPropertiesChanged);
  broker.subscribe('thingEventHappened', onThingEventHappened);
};

TriggerManager.prototype.destoryEvent = function () {
  broker.unsubscribe('propertiesChanged', onPropertiesChanged);
  broker.unsubscribe('thingEventHappened', onThingEventHappened);
};

TriggerManager.prototype.destorySignal = function () {
  broker.unsubscribe("NodeEngine_Event", onScriptEvent);
  broker.unsubscribe("NodeEngine_Config_Event", onGroupConfigUpdate);
  // broker.unsubscribe("NodeEngine_Config_Event", onGroupConfigEvent);
};


TriggerManager.prototype.readConfiguration = function () {
  var confObject = [];
  try {
    var profileStr = fs.readFileSync(this.configPath, "utf-8");
    confObject = JSON.parse(profileStr);
  } catch (e) {
    Log.w(TAG, "configuration Exception = " + e.message);
  }
  return confObject;
};

TriggerManager.prototype.writeConfiguration = function (str) {
  fs.writeFileSync(this.configPath, str);
};

TriggerManager.prototype.getFunctionIdByName = function (serviceName, functionName) {
  var func = triggerScripts.find((item) => {
    var config = item.config[constant.CONFIG_CONFIG];
    return config.serviceName === serviceName && config.functionName === functionName;
  });
  return func && func.functionId;
};

TriggerManager.prototype.isLongLivedFunction = function (functionId) {
  var func = this.config.find((item) => item.functionId === functionId);
  if (func && func.runMode === constant.CONFIG_RUNMODE_LONGLIVE) {
    return true;
  }
  return false;
};

TriggerManager.prototype.getFunctionInfo = function (functionId, callback) {
  var job = this.config.find((item) => item.functionId === functionId);
  var trigger = triggerScripts.find((item) => item.functionId === functionId);
  if (!job || !trigger) {
    callback(new Error(`Function "${functionId}" does not exist: ${job}, ${trigger}`));
    return;
  }
  var info = Object.assign({
    path: job.path,
  }, trigger.config[constant.CONFIG_CONFIG]);
  // For better readability.
  info.runtime = info.fcRuntime;
  callback(null, info);
};

module.exports = TriggerManager;

/**
 * {"ackCode" : int 10201/10204, "groupPath" : dir "yyy", "ackId" : long xx, "jobId" : long id, "executor" : "NodeEngine"}
 * @param info
 */
function onGroupConfigUpdate(info) {
  Log.i(TAG, "onGroupConfigUpdate info = " + JSON.stringify(info));
  var infoObject = info;
  try {
    if (typeof info === "string") {
      infoObject = JSON.parse(info);
    }
  } catch (e) {
    Log.e(TAG, "[EXCEPTION] onGroupConfigUpdate info failed: " + e.message);
    return;
  }
  var confObject = {};
  try {
    var groupPath = path.join(infoObject.groupPath, constant.jobIndex);
    var content = fs.readFileSync(groupPath);
    confObject = JSON.parse(content);
  } catch (e) {
    Log.e(TAG, "configuration Exception = " + e.message);
  }
  for (var script of triggerScripts) {
    script.updateGroupConfig(infoObject.ackCode, confObject);
  }
  var manager = TriggerManager.getInstance();
  var status, message;
  if (Object.keys(confObject).length === 0) {
    if (infoObject.ackCode === 10201) {
      status = TriggerManager.EXEUTE_RESULT.JOB_EXECUTE_FAILED;
      message = "update faild groupConfig";
    } else {
      status = TriggerManager.EXEUTE_RESULT.JOB_DELETE_SUCCESS;//TODO: 删除失败应该返回啥？
      message = "delete success groupConfig";
    }
  } else if (infoObject.ackCode === 10201) {
    status = TriggerManager.EXEUTE_RESULT.JOB_SUCCEED_RUNNING;
    message = "update success groupConfig";
  } else if (infoObject.ackCode === 10204) {
    status = TriggerManager.EXEUTE_RESULT.JOB_DELETE_SUCCESS;
    message = "delete success groupConfig";
  }
  manager.sendJobStatus(infoObject.ackId, infoObject.ackCode, infoObject.jobId, status, infoObject.executor, message, 1, confObject.groupId);
  Log.i(TAG, "onGroupConfigUpdate end.");
};

/**
 * {"type" : "UPDATE/DELETE", "groupId" : long groupId, "groupPath" : "yyy"}
 * @param info
 */
function onGroupConfigEvent(info) {
  Log.i(TAG, "onGroupConfigEvent info = " + JSON.stringify(info));
  //if 不是一键重启 ，return
  var infoObject = info;
  try {
    if (typeof info === "string") {
      infoObject = JSON.parse(info);
    }
  } catch (e) {
    Log.e(TAG, "[EXCEPTION] onGroupConfigEvent info failed: " + e.message);
    return;
  }
  process.groupDir = path.dirname(infoObject.groupPath);
  console.log(TAG, "groupDir = ", process.groupDir);
  var manager = TriggerManager.getInstance();
  if (manager.config) {
    manager.config.forEach(function (item) {
      console.log("onGroupConfigEvent", "item = " + JSON.stringify(item));
      console.log("onGroupConfigEvent", "info = " + JSON.stringify(info));
      if (isGroupMode(item.path + '/' + constant.jobIndex) && (infoObject.groupId === item.groupId || item.parentGroupIds.indexOf(infoObject.groupId) >= 0)) {
        if (infoObject.type === "UPDATE") {
          var newObject = JSON.parse(JSON.stringify(item));
          item.ackCode = TriggerManager.ACTION_DELETE_JOB;
          manager.deleteJob(item, false);
          onScriptEvent(newObject);
        } else {
          fs.unlinkSync(infoObject.groupPath);
          // manager.sendJobStatus(TriggerManager.ACTION_DELETE_JOB, item.jobId, TriggerManager.EXEUTE_RESULT.JOB_DELETE_SUCCESS, item.executor, "delete success");
        }
      }
    });
  }
};

/**
 * 脚本已经下载完成
 * @param  {[Object]} info [已经下载好的script信息，格式如下：
 * {
        "ackCode" : ackCode,
        "jobId" : jobId,
        "type" : type,
        "path" : jobPath,
        "executor" : executor,
        "groupId" : "xxx",
        "parentGroupIds" : ["901", "hotel"]
    }
 ]
 */
function onScriptEvent(info) {
  var manager = TriggerManager.getInstance();
  Log.i(TAG + "onScriptEvent begin... ");
  var object = info;
  try {
    if (typeof info === "string") {
      object = JSON.parse(info);
    }
  } catch (e) {
    Log.e("[EXCEPTION] onScriptEvent info failed: " + e.message);
    return;
  }
  Log.i(TAG, "onScriptEvent newjob = " + JSON.stringify(object, null, 4));
  if (object.ackCode === TriggerManager.ACTION_ADD_JOB) {
    try {
      var jobPath = path.join(object["path"], constant.jobConfig);
      var content = JSON.parse(fs.readFileSync(jobPath));
      object[constant.CONFIG_FUNCTIONID] = content[constant.CONFIG_CONFIG][constant.CONFIG_FUNCTIONID];
      object[constant.CONFIG_RUNMODE] = content[constant.CONFIG_CONFIG][constant.CONFIG_RUNMODE];
      object["sceneId"] = object[constant.CONFIG_FUNCTIONID];
    } catch (err) {
      Log.e(TAG, "onScriptEvent err = " + err);
    }
    manager.config.push(object);
    manager.writeConfiguration(JSON.stringify(manager.config, null, 4));
  }
  manager.trigger(object);
};

function onPropertiesChanged(info) {
  Log.i(TAG, "onPropertiesChanged info = " + JSON.stringify(info));
  console.log(TAG, "triggerScripts.size = " + triggerScripts.length);
  var id = info.id;
  for (var script of triggerScripts) {
    // if (script.triggers.indexOf(id) >= 0) {
    info.eventType = Trigger.TRIGGER_TYPE.propertyChange;
    script.handle(info);
    /*script.exeMain(function (err) {
        if(err) {
            Log.e(TAG, "onPropertiesChanged err = ", err);
            var status = TriggerManager.EXEUTE_RESULT.JOB_EXECUTE_FAILED;
            Log.e(TAG, err);
            TriggerManager.getInstance().sendJobStatus(script.ackCode, script.jobId, status, script.executor, err.toString());
            return;
        }
        Log.i(TAG, "onPropertiesChanged JOB_SUCCEED_RUNNING");
        var status = TriggerManager.EXEUTE_RESULT.JOB_SUCCEED_RUNNING;
        TriggerManager.getInstance().sendJobStatus(script.ackCode, script.jobId, status, script.executor, "execute success");
    });*/
    // }
  }
};


function onThingEventHappened(info) {
  Log.i(TAG, "onThingEventHappened info = " + JSON.stringify(info));
  var id = info.id;
  for (var script of triggerScripts) {
    // if (script.triggers.indexOf(id) >= 0) {
    info.eventType = Trigger.TRIGGER_TYPE.event;
    script.handle(info);
    /*script.exeMain(function (err) {
        if(err) {
            Log.e(TAG, "onThingEventHappened err = ", err);
            var status = TriggerManager.EXEUTE_RESULT.JOB_EXECUTE_FAILED;
            Log.e(TAG, err);
            TriggerManager.getInstance().sendJobStatus(script.ackCode, script.jobId, status, script.executor, err.toString());
            return;
        }
        Log.i(TAG, "onThingEventHappened JOB_SUCCEED_RUNNING");
        var status = TriggerManager.EXEUTE_RESULT.JOB_SUCCEED_RUNNING;
        TriggerManager.getInstance().sendJobStatus(script.ackCode, script.jobId, status, script.executor, "execute success");
    });*/
    // }
  }
};

/**
 * 消息路由过来的消息 JsonObject
 * msg : {
 *    functionId : "",
 *    event : {
 *        payload : {},
 *        topic : " "
 *    }
 * }
 * @param info
 */
function onFilterEvent(info) {
  Log.i(TAG, "onFilterEvent info = " + JSON.stringify(info));
  if (!info || Object.keys(info).length === 0) return;
  for (var script of triggerScripts) {
    if ((script.config)[constant.CONFIG_CONFIG][constant.CONFIG_FUNCTIONID] === info[constant.CONFIG_FUNCTIONID]) {
      script.startFC(info.event, null);
    }
  }
}

function removeOneTrigger(jobId) {
  console.log(TAG, "removeOneTrigger jobId = " + jobId);
  for (var index in triggerScripts) {
    if (triggerScripts[index].jobId === jobId) {
      triggerScripts[index].stopCrontab();
      triggerScripts[index] = null;
      triggerScripts.splice(index, 1);
      break;
    }
  }
  console.log(TAG, "after removeOneTrigger triggerScripts.size = " + triggerScripts.length);
}
