/*
 * Copyright (c) 2017 Alibaba Group Holding Ltd. All rights reversed.
 */
console.log("IFTTT begin...");
var IFTTT = require("./ifttt.js");
var jobScheduler = require("../../scheduler/scheduler.js");
const log = require('../../utils/log');
const broker = require('../../components/broker');
var condition = require("./condition");
const TAG = "IFTTT_index";

var ifttt = undefined;

function setup() {
    log.i(TAG, "IFTTT setup");
    ifttt = new IFTTT(jobScheduler, broker, null);
    ifttt.loadConfig();
    ifttt.eventBus.subscribe("propertiesChanged", onPropertiesChanged);
    ifttt.eventBus.subscribe("IFTTTEngine_Event", onIftttEvent);
    ifttt.eventBus.subscribe("ifttt_timer_event", onIftttTimer);
    ifttt.eventBus.subscribe("thingEventHappened", onThingEventHappened);
    ifttt.eventBus.subscribe("IFTTTEngine_Event_Invoke", onIftttInvoke);
    ifttt.eventBus.subscribe("resetJobs", onResetJobs);
    log.i(TAG, "IFTTT setup end");
    return true;
}

function onPropertiesChanged(event) {
    var info = event;
    log.i("IFTTT onPropertiesChanged: " + JSON.stringify(info) + " size: " + ifttt.scenes.length);
    ifttt.scenes.forEach(function (scene) {
        scene.propertiesChanged(info);
    });
}

function onThingEventHappened(event) {
    var info = event;
    log.i("IFTTT onThingEventHappened: " + JSON.stringify(info) + "ifttt size: " + ifttt.scenes.length);
    ifttt.scenes.forEach(function (scene) {
        scene.eventHappened(info);
    });
}
/**
 * ifttt脚本已经下载完成
 * @param  {[Object]} info [已经下载好的ifttt信息，格式如下：
 * {
        "ackCode" : ackCode,
        "jobId" : jobId,
        "type" : type,
        "path" : jobPath,
        "executor" : executor
    }
]
 * @return {[type]}      [description]
 */
function onIftttEvent(event) {
    var info = event;
    if (IFTTT.ACTION_ADD_SCENE === info.ackCode) {
        // ifttt.refreshIfttt(info);
        ifttt.addScene(info);
    } else if (IFTTT.ACTION_DELETE_SCENE === info.ackCode || IFTTT.COMMAND_DELETE_JOB === info.ackCode) {
        ifttt.deleteScene(info);
        // ifttt.refreshIfttt(info);
    } else {
        log.i("[ERROR] onIftttEvent can not deal with ackCode : " + info.ackCode);
    }
    ifttt.refreshIfttt(info);
}

function onResetJobs() {
    ifttt.deleteAllScene();
}
/**
 * 已经下载安装的ifttt脚本中有timer事件发生，触发检查执行。
 * ifttt_timer_event事件由TimerTrigger.js中发生。
 * @param {Object} info　事件信息，包含scene的id号，标志哪个scene发生了timer事件
 *      {
 *          sceneid: xxxxx
 *      }
 */
function onIftttTimer(event) {
    var info = event;
    log.i(`IFTTT timer happened: ${JSON.stringify(info)}`);
    ifttt.scenes.forEach(function (scene) {
        scene.timerTriggered(info);
    });
}

/**
 * 调用事件非job
 * {
    "attrs":[
		2072, 				//jobId
		"invokeParams..."	//invokeParams
    ],
    "id":1505457082351,
    "reply":false
  }
 {
        "interface" : "actionOnce",
        "payload" : "{\"sceneId\":\"31d6a416b0a446548e95e553f1c67c6b\",\"sceneVersion\":2,\"timestamp\":1528856345194}"
}
 * @param event
 */
function onIftttInvoke(event) {
    log.i(TAG, "onIftttInvoke event = " + JSON.stringify(event));
    var iface = event['interface'];
    var payload = event['payload'];
    if(iface === 'actionOnce') {
        var payload = JSON.parse(event["payload"]);
        var sceneId = payload['sceneId'];
        ifttt.scenes.forEach(function (scene) {
            if(scene.jobInfo.sceneId === sceneId) {
                condition.isConditionsMeeted((scene.sceneObj)['condition'], function (err, satisfy) {
                    if(!err && satisfy) {
                        scene.executeJob();
                    }
                });
                //TODO: 后面再看是否需要回复下Invoke 请求
            }
        });
    }
}

setup();
