var argv = require('yargs').argv;
const Log = require('./developer/log.js');
const constant = require("../utils/constant");
var IFTTT = require('./ifttt/ifttt.js');
var scriptManager = require('./loader/ScriptManager');
var proxy = require('./ws');
var processTimeout = undefined;
const util = require("util");
const broker = require("../components/broker");
const TAG = "Runner";
const RUN_TAG = "[FC_Start_Result]";
var runMode = constant.CONFIG_RUNMODE_ONDEMAND;


function jobDoneCallback(jobId, err) {
    if (processTimeout) {
        clearTimeout(processTimeout);
    }
    if (runMode != constant.CONFIG_RUNMODE_LONGLIVE) {
        Log.LOGI(TAG, 'job ' + jobId + ' finished, err = ', err.toString());
        process.exit(0);
    }
}

try {
    Log.LOGI(`RUNNER start to execute a job: ${argv.type} ${argv.info}`);
    // 1分钟之后，退出当前进程。
    var TimeOver = constant.DEFAULT_TIME_OUT;  // 5s
    var jobtype = argv.type;
    var jobinfo = JSON.parse(argv.info);
    if (jobtype === 'ifttt') {
        var ifttt = new IFTTT(jobinfo, proxy);
        ifttt.execute(true, function (err, status) {
            Log.LOGI(TAG, "Runner IFTTT execute err = " + err + " status = " + status);
            jobDoneCallback(jobinfo.jobId, err);
        });
    }

    if (jobtype === 'script') {
        var jobConfig;
        try {
            jobConfig = JSON.parse(argv.config);
            runMode = jobConfig[constant.CONFIG_CONFIG][constant.CONFIG_RUNMODE];
            var functionId = jobConfig[constant.CONFIG_CONFIG][constant.CONFIG_FUNCTIONID];
            process.env.FUNCTION_ID = functionId;
            process.env.MY_FUNCTION_ID = functionId;
            process.env.ROUTER_FUNCTION_ID = constant.ROUTER_FUNCTION_ID;
            process.env.THING_FUNCTION_ID = constant.THING_FUNCTION_ID;
            process.env.RUN_MODE = runMode;
            if(jobConfig[constant.CONFIG_CONFIG] && (jobConfig[constant.CONFIG_CONFIG]).hasOwnProperty(constant.CONFIG_ENV)) {
                var environment = jobConfig[constant.CONFIG_CONFIG][constant.CONFIG_ENV];
                if(environment && environment[constant.VARIABLES]) {
                    var variables = environment[constant.VARIABLES];
                    for(var key in variables) {
                        (process.env)[key] = variables[key];
                    }
                }
            }
            if (runMode === constant.CONFIG_RUNMODE_LONGLIVE) {
                var result = scriptManager.requireCode(jobinfo, jobConfig);
                if(Object.prototype.toString.call(result) === '[object Error]') {
                    Log.LOGE(TAG, "requireCode Error : " + result.toString() + " path = " + jobinfo.path);
                    Log.LOGM(RUN_TAG, `[Failed][${jobinfo.sceneId}] jobid ${jobinfo.jobId} running status : Fail result:${result.toString()}`);

                } else {
                    Log.LOGM(RUN_TAG, `[Succeeded][${jobinfo.sceneId}] jobid ${jobinfo.jobId} running status : Success result:require success`);
                }
                /*
                * Base 发过来的消息 JsonObject
                * msg : {
                *    event : {
                *        xxx
                *    }
                * }
                */
                broker.subscribe(functionId, function (msg) {
                    Log.LOGI(TAG, "FC LongLive subscribe msg = " + JSON.stringify(msg));
                    if(!msg || Object.keys(msg).length === 0) {
                        Log.LOGE(TAG, "subscribe msg null");
                        return;
                    }
                    jobinfo.event = msg.event;
                    jobinfo.context = msg.context;
                    scriptManager.execute(jobinfo, jobConfig, function (err) {
                        Log.LOGI(TAG, "Runner FC  execute err = " + err);
                        jobDoneCallback(jobinfo.jobId, err);
                    });
                });
            } else {
                TimeOver = jobConfig[constant.CONFIG_CONFIG][constant.CONFIG_TIMEOUT] * 1000;
                // The TimeOver should have a lower bound
                if (TimeOver < 5) {
                    TimeOver = 5;
                }
                scriptManager.execute(jobinfo, jobConfig, function (err) {
                    Log.LOGI(TAG, "Runner FC  execute err = " + err);
                    jobDoneCallback(jobinfo.jobId, err);
                    Log.LOGI("jobId " + jobinfo.jobId + " script exit TimeOver = " + TimeOver);
                });
            }
        } catch (err) {
            Log.LOGE(TAG, "err = " + err);
        }
    }

    processTimeout = setTimeout(() => {
        if (runMode != constant.CONFIG_RUNMODE_LONGLIVE) {
        Log.LOGI(TAG, 'job ' + jobinfo.jobId + ' finished');
        process.exit(0);
        }
    }, TimeOver);
} catch (e) {
    Log.LOGE(`runner execute job failed for: ${e.message}`);
    Log.LOGE(e.stack);
    jobDoneCallback(-1, e);
}
// process.argv.forEach((arg) => {
//     console.log("argv: " + arg);
// })
// console.log("this is runner output.");


function sleep(n) {
    var start = new Date().getTime();
    while(true)  {
        if(new Date().getTime()-start > n) break;
    }
}

process.on("uncaughtException", function (err) {
    Log.LOGE(util.format('[FATAL ERROR] uncaughtException happened: %s', err));
    Log.LOGE(err.stack.replace(/[\r\n]/g, ''));
    Log.LOGI(err.stack);
});
