'use strict'
/** 最新格式文档
 {
     "schema": "http://aliyun/iot/thing/desc/schema",
     "link": "/sys/q408EXte2fy/airCondition/thing/",
     "profile": {
         "productKey": "q408EXte2fy",
         "deviceName": "airCondition"
     },
     "properties": [
         {
             "identifier": "fan_int_property",
             "name": "风扇整数型属性",
             "accessMode": "rw",
             "required": true,
             "dataType": {
                 "type": "int",
                 "specs": {
                     "min": "0",
                     "max": "100",
                     "precise": "0",
                     "unit": "g/ml",
                     "unitName": "毫升"
                 }
             }
         }
     ],

     "events": [
         {
             "identifier": "post",
             "name": "post",
             "type": "info",
             "required": true,
             "desc": "属性上报",
             "method": "thing.event.property.post",
             "outputData": [
                 {
                     "identifier": "fan_int_property",
                     "name": "风扇整数型属性",
                     "dataType": {
                         "type": "int",
                         "specs": {
                             "min": "0",
                             "max": "100",
                             "precise": "0",
                             "unit": "g/ml",
                             "unitName": "毫升"
                         }
                     }
                 },
                 {
                     "identifier": "batch_enum_attr_id",
                     "name": "风扇枚举型属性",
                     "dataType": {
                         "type": "enum",
                         "specs": {
                             "0": "one",
                             "1": "two",
                             "2": "three"
                         }
                     }
                 }
             ]
         }
     ],

     "services": [
             {
                 "name": "upgrade",
                 "identifier": "upgrade",
                 "desc": "升级服务",
                 "inputData": [
                     {
                         "identifier": "model",
                         "name": "模型",
                         "dataType": {
                             "type": "uint",
                             "specs": {
                                 "min": "1",
                                 "max": "100",
                                 "unit": "cm"
                             }
                         }
                     },
                     {
                         "identifier": "mac",
                         "name": "mac号",
                         "dataType": {
                             "type": "text",
                             "specs": {
                                 "length": "255"
                             }
                         }
                     }
                 ],
                 "outputData": [
                     {
                         "identifier": "firmVersion",
                         "name": "固件版本号",
                         "dataType": {
                             "type": "uint",
                             "specs": {
                                 "min": "1",
                                 "max": "100",
                                 "unit": ""
                             }
                         }
                     }
                 ],
                 "method": "thing.service.upgrade"
             }
         ]
 }
 */
var util = require('util');
var proxy = require("../ws");
var DataCenter = require("./DataCenter");
var log = require("../developer/log");
const TAG = "[registry_DynamicDevice] ";

var Device = function (id, tags, callback) {
    var self = this;
    log.LOGI(TAG, "new DynamicDeivce id = " + id);
    if (!id || !callback) {
        var errMsg = `init Device id = ${id}, callback = ${callback}`;
        log.LOGE(TAG, errMsg);
        return callback(new Error(errMsg));
    }
    if (DataCenter.hasThing(id)) {
        log.LOGI(TAG, "DynamicDeivce id " + id + " has been newed.");
        return callback(null, DataCenter.getDeviceObject(id));
    }
    var productKey = id.split("_", 1)[0];
    this.tags = [];
    if (tags && Array.isArray(tags)) {
        for (var tag of tags) {
            var tag = new Tag(tag.key, tag.value);
            if (tag) {
                this.tags.push(tag);
            }
        }
    } else {
        if (tags && tags.key) {
            this.tags.push(new Tag(tags.key, tags.value));
        }
    }
    if (DataCenter.hasProfile(productKey)) {
        var profile = DataCenter.getProfile(productKey);
        this.generateFromProfile(id, profile, callback);
    } else {
        proxy.getDeviceProfile(id, function (err, profile) {
            // console.log(TAG + "id = " + id + "\n profile = " + JSON.stringify(profile, null, 4));
            if (err) {
                return callback(new Error("getDeviceProfileAsync Error, Error = " + err), null);
            }
            DataCenter.addProfile(productKey, profile);
            self.generateFromProfile(id, profile, callback);
        });
    }
};

Device.prototype.get = function (key, callback) {
    if(arguments.length != 2 || typeof arguments[1] != 'function') {
        throw new Error('params error');
        return;
    }
    if(key == null) {
        return callback(new Error('params null error'));
    }
    proxy.getProperty(this.id, [key], function (err, params) {
        console.log(TAG, `getProperty err = ${err}, status = ${JSON.stringify(params)}`);
        if(!err) {
            if(params[key] != undefined) {
                return callback(err, params[key]);
            } else {
                return callback(err, undefined);
            }
        }
        return callback(err);
    });
};

Device.prototype.set = function (key, value, callback) {
    if(arguments.length != 3 || typeof arguments[2] != 'function') {
        throw new Error('params error');
        return;
    }
    if( key == null || value == null) {
        return callback(new Error('params null error'));
    }
    var param = {};
    param[key] = value;
    proxy.setProperty(this.id, param, function (err, status) {
        console.log(TAG, `setProperty err = ${err}, status = ${status}`);
        return callback(err, status);
    });
};

Device.prototype.generateFromProfile = function (id, profile, callback) {
    log.LOGI(TAG, "create DynamicDevice id =" + id);
    if (profile == "" || profile == null || profile === "{}") {
        var errMsg = "Error: profile is invalid. It must be a string or object";
        log.LOGE(TAG, errMsg);
        return callback(new Error(errMsg), null);
    }
    if (typeof profile === 'string') {
        try {
            var pobject = JSON.parse(profile);
        } catch (e) {
            var errMsg = new Error("[EXCEPTION] parse thing profile failed: " + e.message);
            log.LOGE(TAG, errMsg);
            return callback(errMsg, null);
        }
    } else {
        var pobject = profile;
    }
    if (pobject.code === 0 && pobject.params && pobject.params.profile) {
        this.id = id;
        var self = this;
        var realObject = pobject.params.profile;
        // this.generateProperties(realObject.properties);
        this.generateServices(realObject.services);
        DataCenter.addDevice(self.id, self);
        return callback(null, self);
    } else {
        var errMsg = new Error(id + " DynamicDevice profile Error, profile = " + JSON.stringify(pobject));
        log.LOGE(TAG, errMsg);
        return callback(errMsg, null);
    }
};

Device.prototype.generateProperties = function (properties) {
    if (!properties) {
        log.LOGE(TAG, "DynamicDevice properties = " + properties);
        return;
    }
    var dict = {};
    properties.forEach((property) => {
        var pname = property.identifier;
        var amode = property.accessMode;

        dict[pname] = {};
        dict[pname].configurable = true;
        dict[pname].enumerable = true;

        if (amode.includes('r') || !amode) {
            dict[pname].get = function () {
                console.log(TAG + util.format("get property %s.%s ", this.id, pname));
                return proxy.getProperty(this.id, [pname]);
            }
        }

        if (amode.includes('w') || !amode) {
            dict[pname].set = function (value) {
                console.log(TAG + util.format("set property %s.%s = %s", this.id, pname, value));
                var param = {};
                param[pname] = value;
                return proxy.setProperty(this.id, param);
            }
        }
    });
    Object.defineProperties(this, dict);
};

Device.prototype.generateServices = function (services) {
    if (!services) {
        log.LOGE(TAG, "DynamicDevice services = " + services);
        return;
    }
    services.forEach((service) => {
        if(service.identifier === "set" || service.identifier === "get") {
            return;//forEach 中return 只表示跳过本轮循环而非跳出循环
        }
        Object.defineProperty(this, service.identifier, {
            value: function () {
                console.log(TAG + "Function arguments size = " + arguments.length);
                var inputs = service.inputData;
                var outputs = service.outputData;
                var param = {};
                if (inputs != null) {
                    for (var i = 0; i < inputs.length; i++) {
                        var key = inputs[i].identifier;
                        if(key) {
                            param[key] = arguments[i];
                        }
                    }
                }
                log.LOGI(TAG, `Function '${service.identifier}' arguments size = ${arguments.length} inputs.length = ${inputs.length}`);
                if (arguments.length > inputs.length) {
                    var callback = arguments[inputs.length];
                    log.LOGI(TAG, util.format("Call method: (%s, %s, %s, %s)", this.id, service.identifier, JSON.stringify(param), callback));
                    var self = this;
                    proxy.callService(this.id, service.identifier, param, function (err, value) {
                        callback(err, value);
                    });
                } else {
                    var isEmpty = (Object.keys(param).length === 0);
                    if (isEmpty) {
                        log.LOGI(TAG, util.format("Call method: (%s, %s)", this.id, service.identifier));
                        return proxy.callService(this.id, service.identifier, "", () => {
                        });
                    } else {
                        log.LOGI(TAG, util.format("Call method: (%s, %s, %s)", this.id, service.identifier, JSON.stringify(param)));
                        return proxy.callService(this.id, service.identifier, param, () => {
                        });
                    }
                }
            }
        });
    });
};

Device.prototype.getTagList = function () {
    return this.tags;
};

Device.prototype.setTagList = function (list) {
    this.tags = list;
};

Device.prototype.getTag = function (tagName) {
    if (this.tags) {
        for(var tag of this.tags) {
            if (tag.key === tagName) {
                return tag;
            }
        };
    }
};

Device.prototype.setTag = function (tagName, tagValue) {
    if (this.tags) {
        for(var tag of this.tags) {
            if (tag.key === tagName) {
                tag.value = tagValue;
                return true;
            }
        };
    }
    return false;
};

Device.prototype.setAliasName = function (name) {
    this.name = name;
};

Device.prototype.getName = function () {
    return this.name;
};

var Tag = function (key, value) {
    if (!key) {
        return null;
    }
    this.key = key;
    this.value = value;
};

module.exports = Device;
