'use strict';
var DDevice = require("./DynamicDevice");
var Log = require("../developer/log");
const TAG = "[registry_registerDevice] ";

var Register = function () {

};

/**
 * @description 添加设备入口
 * @param {String} id
 * @return {DynamicDevice} device 对象
 **/
Register.prototype.createThing = function (id, tags, callback) {
    Log.LOGI(TAG, "begin createThing id = " + id);
    new DDevice(id, tags, function (err, thing) {
        return callback(err, thing);
    });
};

/**
 * @description 增加一个数组列表的设备
 * @param {Array} idList
 * @return {Array} devicesList
 */
Register.prototype.createThings = function (idList, callback) {
    Log.LOGI(TAG, "begin createThings list = " + JSON.stringify(idList));
    var result = [];
    var size = idList.length;
    if (!idList || size == 0) {
        return callback(false, result);
    }
    var i = 0;
    var self = this;
    idList.forEach(function (id) {
        self.createThing(id, null, function(err, thing) {
            if(err) {
                return callback(err);
            }
            result.push(thing);
            i++;
            if(i === size) {
                return callback(false, result);
            }
        });
    });
};

/**
 * 判断实例化的设备对象是不是有效设备
 * @param thing
 * @return {boolean}
 */
Register.prototype.isValid = function (thing) {
    if (!thing || JSON.stringify(thing) === "{}") {
        return false;
    }
    return true;
};

module.exports = new Register();
