#!/usr/bin/env python
#-*- coding: utf-8 -*-
import os
import config
import ws_client
import sync
import fc_error
import utils
import logging


class IPC_Wrapper(object):
    _instance = None
    def __init__(self, functionId, auth):
        self.ws = ws_client.ws
        self.functionId = functionId
        self.auth = auth
        self.getParamSync = sync.SyncMsg_Event()
        self.postResultSync = sync.SyncMsg_Event()
        self.postHandlerErrSync = sync.SyncMsg_Event()
        self.invokeTaskSync = sync.SyncMsg_Event()
        self.getTaskResultSync = sync.SyncMsg_Event()
        self.getCredentialSync = sync.SyncMsg_Event()

    def __new__(self, *args, **kw):
        if not self._instance:
            self._instance = super(IPC_Wrapper, self).__new__(self)
        return self._instance

    def _getParamResult_cb(self, status, data):
        syncMsg = {}
        syncMsg["state"] = False
        syncMsg["msg"] = None
        try:
            logging.debug("staus:%s", status)
            if status == "success":
                syncMsg["state"] = True
                syncMsg["msg"] = data
            else:
                syncMsg["msg"] = ''
                syncMsg["state"] = False
        except:
            syncMsg["msg"] = ''
            syncMsg["state"] = False
        self.getParamSync.set(syncMsg)

    def get_param(self):
        options = {"method": 'GET',
                   "path": utils.getTaskRequestPath(self.functionId),
                   "headers": {
                       config.HEADER_AUTH_TOKEN: self.auth}
                  }

        self.getParamSync.clear()
        reply_cb = self._getParamResult_cb
        self.ws.request(options, reply_cb)
        syncMsg = self.getParamSync.wait()
        if (None == syncMsg):
            raise fc_error.WSConnException(msg="get param timeout")
            return None
        logging.debug("get param:%s", syncMsg)
        return syncMsg

    def _postTaskResult_cb(self, status, data):
        syncMsg = {}
        syncMsg["state"] = False
        syncMsg["msg"] = None
        try:
            logging.debug("staus:%s", status)
            if status == "success":
                syncMsg["state"] = True
                syncMsg["msg"] = data
            else:
                syncMsg["msg"] = ''
                syncMsg["state"] = False
        except:
            syncMsg["msg"] = ''
            syncMsg["state"] = False
        self.postResultSync.set(syncMsg)

    def postTaskResult(self, invocationId, result):
        options = { "method": 'POST',
                    "path": utils.getTaskRequestPath(self.functionId),
                    "headers": {
                        config.HEADER_AUTH_TOKEN: self.auth,
                        config.HEADER_INVOCATION_ID: invocationId},
                    "body": result
                  }

        self.postResultSync.clear()
        reply_cb = self._postTaskResult_cb
        self.ws.request(options, reply_cb)
        syncMsg = self.postResultSync.wait(config.MERHOD_SYNC_TIMEOUT)
        if (None == syncMsg):
            raise fc_error.WSConnException(msg="post task result timeout")
            return None
        logging.debug("postTaskResult:%s",syncMsg)
        return syncMsg

    def _postHandlerErr_cb(self, status, data):
        syncMsg = {}
        syncMsg["state"] = False
        syncMsg["msg"] = None
        try:
            logging.debug("staus:%s", status)
            if status == "success":
                syncMsg["state"] = True
                syncMsg["msg"] = data
            else:
                syncMsg["msg"] = ''
                syncMsg["state"] = False
        except:
            syncMsg["msg"] = ''
            syncMsg["state"] = False
        self.postHandlerErrSync.set(syncMsg)

    def postHandlerErr(self, invocationId, error):
        options = { "method": 'POST',
                    "path": utils.getTaskRequestPath(self.functionId),
                    "headers": {
                        config.HEADER_AUTH_TOKEN: self.auth,
                        config.HEADER_INVOCATION_ID: invocationId,
                        config.HEADER_FUNCTION_ERROR: 'Handled'},
                    "body": error}
        self.postHandlerErrSync.clear()
        reply_cb = self._postTaskResult_cb
        self.ws.request(options, reply_cb)
        syncMsg = self.postHandlerErrSync.wait(config.MERHOD_SYNC_TIMEOUT)
        if (None == syncMsg):
            raise fc_error.WSConnException(msg="post task result timeout")
            return None
        logging.debug("postHandlerErr:%s", syncMsg)
        return syncMsg
        
    def _invokeTask_cb(self, status, data):
        syncMsg = {}
        syncMsg["state"] = False
        syncMsg["msg"] = None
        try:
            logging.debug("staus:%s", status)
            if status == "success":
                syncMsg["state"] = True
                syncMsg["msg"] = data
            else:
                syncMsg["msg"] = ''
                syncMsg["state"] = False
        except:
            syncMsg["msg"] = ''
            syncMsg["state"] = False
        self.invokeTaskSync.set(syncMsg)

    def invokeTask(self, functionId, invokerContext, invocationType, payload):
        options = {"method": 'POST',
                   "path": utils.getRequestPath(functionId),
                   "headers": {
                       config.HEADER_AUTH_TOKEN: self.auth,
                       config.HEADER_INVOKER_CONTEXT: invokerContext,
                       config.HEADER_INVOCATION_TYPE: invocationType},
                   "body": payload}
        self.invokeTaskSync.clear()
        reply_cb = self._invokeTask_cb
        self.ws.request(options, reply_cb)
        syncMsg = self.invokeTaskSync.wait(config.MERHOD_SYNC_TIMEOUT)
        if (None == syncMsg):
            raise fc_error.WSConnException(msg="set property timeout")
            return None
        logging.debug("invokeTask:%s", syncMsg)
        return syncMsg

    def _getTaskResult_cb(self, status, data):
        syncMsg = {}
        syncMsg["state"] = False
        syncMsg["msg"] = None
        try:
            logging.debug("staus:%s", status)
            if status == "success":
                syncMsg["state"] = True
                syncMsg["msg"] = data
            else:
                syncMsg["msg"] = ''
                syncMsg["state"] = False
        except:
            syncMsg["msg"] = ''
            syncMsg["state"] = False
        self.getTaskResultSync.set(syncMsg)

    def getTaskResult(self, functionId, invocationId):
        options = { "method": 'GET',
                    "path": utils.getRequestPath(functionId),
                    "headers": {
                        config.HEADER_AUTH_TOKEN: self.auth,
                        config.HEADER_INVOCATION_ID: invocationId}
                  }
        self.getTaskResultSync.clear()
        reply_cb = self._getTaskResult_cb
        self.ws.request(options, reply_cb)
        syncMsg = self.getTaskResultSync.wait()
        if (None == syncMsg):
            raise fc_error.WSConnException(msg="post task result timeout")
            return None
        logging.debug("getTaskResult:%s", syncMsg)
        return syncMsg

    def _getCredential_cb(self, status, data):
        syncMsg = {}
        syncMsg["state"] = False
        syncMsg["msg"] = None
        try:
            logging.debug("staus:%s", status)
            if status == "success":
                syncMsg["state"] = True
                syncMsg["msg"] = data
            else:
                syncMsg["msg"] = ''
                syncMsg["state"] = False
        except:
            syncMsg["msg"] = ''
            syncMsg["state"] = False
        self.getCredentialSync.set(syncMsg)

    def get_credential(self, uri):
        options = {
            "method": 'GET',
            "path": uri,
            "headers": {config.HEADER_AUTH_TOKEN: self.auth}}

        self.getCredentialSync.clear()
        reply_cb = self._getCredential_cb
        self.ws.request(options, reply_cb)
        syncMsg = self.getCredentialSync.wait()
        if (None == syncMsg):
            raise fc_error.WSConnException(msg="get param timeout")
            return None
        logging.debug("get_credential:%s", syncMsg)
        return syncMsg

def get_ipc_wrapper():
    return IPC_Wrapper(os.environ.get("FUNCTION_ID"), os.environ.get("AUTH_TOKEN"))

ipc = get_ipc_wrapper()
